@@ -1,8 +1,10 @@
 from io import BytesIO
 
+import pytest
+
 from PIL import Image
 
-from .helper import skip_unless_feature
+from .helper import mark_if_feature_version, skip_unless_feature
 
 pytestmark = [
     skip_unless_feature("webp"),
@@ -30,6 +32,18 @@ def test_read_exif_metadata():
             assert exif_data == expected_exif
 
 
+def test_read_exif_metadata_without_prefix():
+    with Image.open("Tests/images/flower2.webp") as im:
+        # Assert prefix is not present
+        assert im.info["exif"][:6] != b"Exif\x00\x00"
+
+        exif = im.getexif()
+        assert exif[305] == "Adobe Photoshop CS6 (Macintosh)"
+
+
+@mark_if_feature_version(
+    pytest.mark.valgrind_known_error, "libjpeg_turbo", "2.0", reason="Known Failing"
+)
 def test_write_exif_metadata():
     file_path = "Tests/images/flower.jpg"
     test_buffer = BytesIO()
@@ -41,9 +55,7 @@ def test_write_exif_metadata():
     test_buffer.seek(0)
     with Image.open(test_buffer) as webp_image:
         webp_exif = webp_image.info.get("exif", None)
-    assert webp_exif
-    if webp_exif:
-        assert webp_exif == expected_exif, "WebP EXIF didn't match"
+    assert webp_exif == expected_exif[6:], "WebP EXIF didn't match"
 
 
 def test_read_icc_profile():
@@ -62,6 +74,9 @@ def test_read_icc_profile():
             assert icc == expected_icc
 
 
+@mark_if_feature_version(
+    pytest.mark.valgrind_known_error, "libjpeg_turbo", "2.0", reason="Known Failing"
+)
 def test_write_icc_metadata():
     file_path = "Tests/images/flower2.jpg"
     test_buffer = BytesIO()
@@ -79,6 +94,9 @@ def test_write_icc_metadata():
         assert webp_icc_profile == expected_icc_profile, "Webp ICC didn't match"
 
 
+@mark_if_feature_version(
+    pytest.mark.valgrind_known_error, "libjpeg_turbo", "2.0", reason="Known Failing"
+)
 def test_read_no_exif():
     file_path = "Tests/images/flower.jpg"
     test_buffer = BytesIO()
