@@ -27,15 +27,21 @@ def test_sanity():
         "HSV",
     )
 
-    for mode in modes:
-        im = hopper(mode)
-        for mode in modes:
-            convert(im, mode)
+    for input_mode in modes:
+        im = hopper(input_mode)
+        for output_mode in modes:
+            convert(im, output_mode)
 
         # Check 0
-        im = Image.new(mode, (0, 0))
-        for mode in modes:
-            convert(im, mode)
+        im = Image.new(input_mode, (0, 0))
+        for output_mode in modes:
+            convert(im, output_mode)
+
+
+def test_unsupported_conversion():
+    im = hopper()
+    with pytest.raises(ValueError):
+        im.convert("INVALID")
 
 
 def test_default():
@@ -222,6 +228,37 @@ def test_p_la():
     assert_image_similar(alpha, comparable, 5)
 
 
+def test_p2pa_alpha():
+    with Image.open("Tests/images/tiny.png") as im:
+        assert im.mode == "P"
+
+        im_pa = im.convert("PA")
+    assert im_pa.mode == "PA"
+
+    im_a = im_pa.getchannel("A")
+    for x in range(4):
+        alpha = 255 if x > 1 else 0
+        for y in range(4):
+            assert im_a.getpixel((x, y)) == alpha
+
+
+def test_p2pa_palette():
+    with Image.open("Tests/images/tiny.png") as im:
+        im_pa = im.convert("PA")
+    assert im_pa.getpalette() == im.getpalette()
+
+
+@pytest.mark.parametrize("mode", ("RGB", "RGBA", "RGBX"))
+def test_rgb_lab(mode):
+    im = Image.new(mode, (1, 1))
+    converted_im = im.convert("LAB")
+    assert converted_im.getpixel((0, 0)) == (0, 128, 128)
+
+    im = Image.new("LAB", (1, 1), (255, 0, 0))
+    converted_im = im.convert(mode)
+    assert converted_im.getpixel((0, 0))[:3] == (0, 255, 255)
+
+
 def test_matrix_illegal_conversion():
     # Arrange
     im = hopper("CMYK")
@@ -254,36 +291,33 @@ def test_matrix_wrong_mode():
         im.convert(mode="L", matrix=matrix)
 
 
-def test_matrix_xyz():
-    def matrix_convert(mode):
-        # Arrange
-        im = hopper("RGB")
-        im.info["transparency"] = (255, 0, 0)
-        # fmt: off
-        matrix = (
-            0.412453, 0.357580, 0.180423, 0,
-            0.212671, 0.715160, 0.072169, 0,
-            0.019334, 0.119193, 0.950227, 0)
-        # fmt: on
-        assert im.mode == "RGB"
+@pytest.mark.parametrize("mode", ("RGB", "L"))
+def test_matrix_xyz(mode):
+    # Arrange
+    im = hopper("RGB")
+    im.info["transparency"] = (255, 0, 0)
+    # fmt: off
+    matrix = (
+        0.412453, 0.357580, 0.180423, 0,
+        0.212671, 0.715160, 0.072169, 0,
+        0.019334, 0.119193, 0.950227, 0)
+    # fmt: on
+    assert im.mode == "RGB"
 
-        # Act
-        # Convert an RGB image to the CIE XYZ colour space
-        converted_im = im.convert(mode=mode, matrix=matrix)
+    # Act
+    # Convert an RGB image to the CIE XYZ colour space
+    converted_im = im.convert(mode=mode, matrix=matrix)
 
-        # Assert
-        assert converted_im.mode == mode
-        assert converted_im.size == im.size
-        with Image.open("Tests/images/hopper-XYZ.png") as target:
-            if converted_im.mode == "RGB":
-                assert_image_similar(converted_im, target, 3)
-                assert converted_im.info["transparency"] == (105, 54, 4)
-            else:
-                assert_image_similar(converted_im, target.getchannel(0), 1)
-                assert converted_im.info["transparency"] == 105
-
-    matrix_convert("RGB")
-    matrix_convert("L")
+    # Assert
+    assert converted_im.mode == mode
+    assert converted_im.size == im.size
+    with Image.open("Tests/images/hopper-XYZ.png") as target:
+        if converted_im.mode == "RGB":
+            assert_image_similar(converted_im, target, 3)
+            assert converted_im.info["transparency"] == (105, 54, 4)
+        else:
+            assert_image_similar(converted_im, target.getchannel(0), 1)
+            assert converted_im.info["transparency"] == 105
 
 
 def test_matrix_identity():
