@@ -37,19 +37,12 @@
 #define MAX(a, b) (a) > (b) ? (a) : (b)
 #define MIN(a, b) (a) < (b) ? (a) : (b)
 
-#define CLIP16(v) ((v) <= -32768 ? -32768 : (v) >= 32767 ? 32767 : (v))
+#define CLIP16(v) ((v) <= 0 ? 0 : (v) >= 65535 ? 65535 : (v))
 
 /* ITU-R Recommendation 601-2 (assuming nonlinear RGB) */
 #define L(rgb) ((INT32)(rgb)[0] * 299 + (INT32)(rgb)[1] * 587 + (INT32)(rgb)[2] * 114)
 #define L24(rgb) ((rgb)[0] * 19595 + (rgb)[1] * 38470 + (rgb)[2] * 7471 + 0x8000)
 
-#ifndef round
-double
-round(double x) {
-    return floor(x + 0.5);
-}
-#endif
-
 /* ------------------- */
 /* 1 (bit) conversions */
 /* ------------------- */
@@ -991,115 +984,124 @@ static struct {
 /* ------------------- */
 
 static void
-p2bit(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+p2bit(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     /* FIXME: precalculate greyscale palette? */
     for (x = 0; x < xsize; x++) {
-        *out++ = (L(&palette[in[x] * 4]) >= 128000) ? 255 : 0;
+        *out++ = (L(&palette->palette[in[x] * 4]) >= 128000) ? 255 : 0;
     }
 }
 
 static void
-pa2bit(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+pa2bit(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     /* FIXME: precalculate greyscale palette? */
     for (x = 0; x < xsize; x++, in += 4) {
-        *out++ = (L(&palette[in[0] * 4]) >= 128000) ? 255 : 0;
+        *out++ = (L(&palette->palette[in[0] * 4]) >= 128000) ? 255 : 0;
     }
 }
 
 static void
-p2l(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+p2l(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     /* FIXME: precalculate greyscale palette? */
     for (x = 0; x < xsize; x++) {
-        *out++ = L(&palette[in[x] * 4]) / 1000;
+        *out++ = L24(&palette->palette[in[x] * 4]) >> 16;
     }
 }
 
 static void
-pa2l(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+pa2l(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     /* FIXME: precalculate greyscale palette? */
     for (x = 0; x < xsize; x++, in += 4) {
-        *out++ = L(&palette[in[0] * 4]) / 1000;
+        *out++ = L24(&palette->palette[in[0] * 4]) >> 16;
     }
 }
 
 static void
-p2pa(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+pa2p(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
+    for (x = 0; x < xsize; x++, in += 4) {
+        *out++ = in[0];
+    }
+}
+
+static void
+p2pa(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
+    int x;
+    int rgb = strcmp(palette->mode, "RGB");
     for (x = 0; x < xsize; x++, in++) {
-        const UINT8 *rgba = &palette[in[0]];
+        const UINT8 *rgba = &palette->palette[in[0] * 4];
         *out++ = in[0];
         *out++ = in[0];
         *out++ = in[0];
-        *out++ = rgba[3];
+        *out++ = rgb == 0 ? 255 : rgba[3];
     }
 }
 
 static void
-p2la(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+p2la(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     /* FIXME: precalculate greyscale palette? */
     for (x = 0; x < xsize; x++, out += 4) {
-        const UINT8 *rgba = &palette[*in++ * 4];
-        out[0] = out[1] = out[2] = L(rgba) / 1000;
+        const UINT8 *rgba = &palette->palette[*in++ * 4];
+        out[0] = out[1] = out[2] = L24(rgba) >> 16;
         out[3] = rgba[3];
     }
 }
 
 static void
-pa2la(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+pa2la(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     /* FIXME: precalculate greyscale palette? */
     for (x = 0; x < xsize; x++, in += 4, out += 4) {
-        out[0] = out[1] = out[2] = L(&palette[in[0] * 4]) / 1000;
+        out[0] = out[1] = out[2] = L24(&palette->palette[in[0] * 4]) >> 16;
         out[3] = in[3];
     }
 }
 
 static void
-p2i(UINT8 *out_, const UINT8 *in, int xsize, const UINT8 *palette) {
+p2i(UINT8 *out_, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     for (x = 0; x < xsize; x++, out_ += 4) {
-        INT32 v = L(&palette[in[x] * 4]) / 1000;
+        INT32 v = L24(&palette->palette[in[x] * 4]) >> 16;
         memcpy(out_, &v, sizeof(v));
     }
 }
 
 static void
-pa2i(UINT8 *out_, const UINT8 *in, int xsize, const UINT8 *palette) {
+pa2i(UINT8 *out_, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     INT32 *out = (INT32 *)out_;
     for (x = 0; x < xsize; x++, in += 4) {
-        *out++ = L(&palette[in[0] * 4]) / 1000;
+        *out++ = L24(&palette->palette[in[0] * 4]) >> 16;
     }
 }
 
 static void
-p2f(UINT8 *out_, const UINT8 *in, int xsize, const UINT8 *palette) {
+p2f(UINT8 *out_, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     for (x = 0; x < xsize; x++, out_ += 4) {
-        FLOAT32 v = L(&palette[in[x] * 4]) / 1000.0F;
+        FLOAT32 v = L(&palette->palette[in[x] * 4]) / 1000.0F;
         memcpy(out_, &v, sizeof(v));
     }
 }
 
 static void
-pa2f(UINT8 *out_, const UINT8 *in, int xsize, const UINT8 *palette) {
+pa2f(UINT8 *out_, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     FLOAT32 *out = (FLOAT32 *)out_;
     for (x = 0; x < xsize; x++, in += 4) {
-        *out++ = (float)L(&palette[in[0] * 4]) / 1000.0F;
+        *out++ = (float)L(&palette->palette[in[0] * 4]) / 1000.0F;
     }
 }
 
 static void
-p2rgb(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+p2rgb(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     for (x = 0; x < xsize; x++) {
-        const UINT8 *rgb = &palette[*in++ * 4];
+        const UINT8 *rgb = &palette->palette[*in++ * 4];
         *out++ = rgb[0];
         *out++ = rgb[1];
         *out++ = rgb[2];
@@ -1108,10 +1110,10 @@ p2rgb(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
 }
 
 static void
-pa2rgb(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+pa2rgb(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     for (x = 0; x < xsize; x++, in += 4) {
-        const UINT8 *rgb = &palette[in[0] * 4];
+        const UINT8 *rgb = &palette->palette[in[0] * 4];
         *out++ = rgb[0];
         *out++ = rgb[1];
         *out++ = rgb[2];
@@ -1120,30 +1122,30 @@ pa2rgb(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
 }
 
 static void
-p2hsv(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+p2hsv(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     for (x = 0; x < xsize; x++, out += 4) {
-        const UINT8 *rgb = &palette[*in++ * 4];
+        const UINT8 *rgb = &palette->palette[*in++ * 4];
         rgb2hsv_row(out, rgb);
         out[3] = 255;
     }
 }
 
 static void
-pa2hsv(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+pa2hsv(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     for (x = 0; x < xsize; x++, in += 4, out += 4) {
-        const UINT8 *rgb = &palette[in[0] * 4];
+        const UINT8 *rgb = &palette->palette[in[0] * 4];
         rgb2hsv_row(out, rgb);
         out[3] = 255;
     }
 }
 
 static void
-p2rgba(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+p2rgba(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     for (x = 0; x < xsize; x++) {
-        const UINT8 *rgba = &palette[*in++ * 4];
+        const UINT8 *rgba = &palette->palette[*in++ * 4];
         *out++ = rgba[0];
         *out++ = rgba[1];
         *out++ = rgba[2];
@@ -1152,10 +1154,10 @@ p2rgba(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
 }
 
 static void
-pa2rgba(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+pa2rgba(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     int x;
     for (x = 0; x < xsize; x++, in += 4) {
-        const UINT8 *rgb = &palette[in[0] * 4];
+        const UINT8 *rgb = &palette->palette[in[0] * 4];
         *out++ = rgb[0];
         *out++ = rgb[1];
         *out++ = rgb[2];
@@ -1164,25 +1166,25 @@ pa2rgba(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
 }
 
 static void
-p2cmyk(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+p2cmyk(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     p2rgb(out, in, xsize, palette);
     rgb2cmyk(out, out, xsize);
 }
 
 static void
-pa2cmyk(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+pa2cmyk(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     pa2rgb(out, in, xsize, palette);
     rgb2cmyk(out, out, xsize);
 }
 
 static void
-p2ycbcr(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+p2ycbcr(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     p2rgb(out, in, xsize, palette);
     ImagingConvertRGB2YCbCr(out, out, xsize);
 }
 
 static void
-pa2ycbcr(UINT8 *out, const UINT8 *in, int xsize, const UINT8 *palette) {
+pa2ycbcr(UINT8 *out, const UINT8 *in, int xsize, ImagingPalette palette) {
     pa2rgb(out, in, xsize, palette);
     ImagingConvertRGB2YCbCr(out, out, xsize);
 }
@@ -1192,7 +1194,7 @@ frompalette(Imaging imOut, Imaging imIn, const char *mode) {
     ImagingSectionCookie cookie;
     int alpha;
     int y;
-    void (*convert)(UINT8 *, const UINT8 *, int, const UINT8 *);
+    void (*convert)(UINT8 *, const UINT8 *, int, ImagingPalette);
 
     /* Map palette image to L, RGB, RGBA, or CMYK */
 
@@ -1208,6 +1210,8 @@ frompalette(Imaging imOut, Imaging imIn, const char *mode) {
         convert = alpha ? pa2l : p2l;
     } else if (strcmp(mode, "LA") == 0) {
         convert = alpha ? pa2la : p2la;
+    } else if (strcmp(mode, "P") == 0) {
+        convert = pa2p;
     } else if (strcmp(mode, "PA") == 0) {
         convert = p2pa;
     } else if (strcmp(mode, "I") == 0) {
@@ -1216,9 +1220,7 @@ frompalette(Imaging imOut, Imaging imIn, const char *mode) {
         convert = alpha ? pa2f : p2f;
     } else if (strcmp(mode, "RGB") == 0) {
         convert = alpha ? pa2rgb : p2rgb;
-    } else if (strcmp(mode, "RGBA") == 0) {
-        convert = alpha ? pa2rgba : p2rgba;
-    } else if (strcmp(mode, "RGBX") == 0) {
+    } else if (strcmp(mode, "RGBA") == 0 || strcmp(mode, "RGBX") == 0) {
         convert = alpha ? pa2rgba : p2rgba;
     } else if (strcmp(mode, "CMYK") == 0) {
         convert = alpha ? pa2cmyk : p2cmyk;
@@ -1234,6 +1236,10 @@ frompalette(Imaging imOut, Imaging imIn, const char *mode) {
     if (!imOut) {
         return NULL;
     }
+    if (strcmp(mode, "P") == 0 || strcmp(mode, "PA") == 0) {
+        ImagingPaletteDelete(imOut->palette);
+        imOut->palette = ImagingPaletteDuplicate(imIn->palette);
+    }
 
     ImagingSectionEnter(&cookie);
     for (y = 0; y < imIn->ysize; y++) {
@@ -1241,7 +1247,7 @@ frompalette(Imaging imOut, Imaging imIn, const char *mode) {
             (UINT8 *)imOut->image[y],
             (UINT8 *)imIn->image[y],
             imIn->xsize,
-            imIn->palette->palette);
+            imIn->palette);
     }
     ImagingSectionLeave(&cookie);
 
@@ -1448,7 +1454,7 @@ topalette(
 }
 
 static Imaging
-tobilevel(Imaging imOut, Imaging imIn, int dither) {
+tobilevel(Imaging imOut, Imaging imIn) {
     ImagingSectionCookie cookie;
     int x, y;
     int *errors;
@@ -1575,7 +1581,7 @@ convert(
     }
 
     if (dither && strcmp(mode, "1") == 0) {
-        return tobilevel(imOut, imIn, dither);
+        return tobilevel(imOut, imIn);
     }
 
     /* standard conversion machinery */
@@ -1594,9 +1600,8 @@ convert(
 #ifdef notdef
         return (Imaging)ImagingError_ValueError("conversion not supported");
 #else
-        static char buf[256];
-        /* FIXME: may overflow if mode is too large */
-        sprintf(buf, "conversion from %s to %s not supported", imIn->mode, mode);
+        static char buf[100];
+        snprintf(buf, 100, "conversion from %.10s to %.10s not supported", imIn->mode, mode);
         return (Imaging)ImagingError_ValueError(buf);
 #endif
     }
@@ -1636,29 +1641,15 @@ ImagingConvertTransparent(Imaging imIn, const char *mode, int r, int g, int b) {
         return (Imaging)ImagingError_ModeError();
     }
 
-    if (!((strcmp(imIn->mode, "RGB") == 0 || strcmp(imIn->mode, "1") == 0 ||
-           strcmp(imIn->mode, "I") == 0 || strcmp(imIn->mode, "L") == 0) &&
-          strcmp(mode, "RGBA") == 0))
-#ifdef notdef
-    {
-        return (Imaging)ImagingError_ValueError("conversion not supported");
-    }
-#else
-    {
-        static char buf[256];
-        /* FIXME: may overflow if mode is too large */
-        sprintf(
-            buf,
-            "conversion from %s to %s not supported in convert_transparent",
-            imIn->mode,
-            mode);
-        return (Imaging)ImagingError_ValueError(buf);
-    }
-#endif
-
-    if (strcmp(imIn->mode, "RGB") == 0) {
+    if (strcmp(imIn->mode, "RGB") == 0 && strcmp(mode, "RGBA") == 0) {
         convert = rgb2rgba;
-    } else {
+    } else if ((strcmp(imIn->mode, "1") == 0 ||
+                strcmp(imIn->mode, "I") == 0 ||
+                strcmp(imIn->mode, "L") == 0
+               ) && (
+                strcmp(mode, "RGBA") == 0 ||
+                strcmp(mode, "LA") == 0
+               )) {
         if (strcmp(imIn->mode, "1") == 0) {
             convert = bit2rgb;
         } else if (strcmp(imIn->mode, "I") == 0) {
@@ -1667,6 +1658,15 @@ ImagingConvertTransparent(Imaging imIn, const char *mode, int r, int g, int b) {
             convert = l2rgb;
         }
         g = b = r;
+    } else {
+        static char buf[100];
+        snprintf(
+            buf,
+            100,
+            "conversion from %.10s to %.10s not supported in convert_transparent",
+            imIn->mode,
+            mode);
+        return (Imaging)ImagingError_ValueError(buf);
     }
 
     imOut = ImagingNew2Dirty(mode, imOut, imIn);
